#include <scene_rdl2/scene/rdl2/rdl2.h>
using namespace scene_rdl2::rdl2;
RDL2_DSO_ATTR_DECLARE_NS(ImageMap_attr)
AttributeKey<Bool> attrWrapAround;
AttributeKey<String> attrTexture;
AttributeKey<Bool> attrUseDefaultColor;
AttributeKey<Rgb> attrDefaultColor;
AttributeKey<Int> attrTextureEnum;
AttributeKey<Bool> attrAlphaOnly;
AttributeKey<Vec3f> attrInputTextureCoordinate;
AttributeKey<Float> attrMipBias;
AttributeKey<Int> attrGamma;
AttributeKey<Vec2f> attrOffset;
AttributeKey<Vec2f> attrScale;
AttributeKey<Float> attrRotationAngle;
AttributeKey<Vec2f> attrRotationCenter;
AttributeKey<Bool> attrSaturationEnabled;
AttributeKey<Rgb> attrSaturation;
AttributeKey<Bool> attrContrastEnabled;
AttributeKey<Rgb> attrContrast;
AttributeKey<Bool> attrGammaEnabled;
AttributeKey<Rgb> attrGammaAdjust;
AttributeKey<Bool> attrGainOffsetEnabled;
AttributeKey<Rgb> attrGain;
AttributeKey<Rgb> attrOffsetAdjust;
AttributeKey<Bool> attrTMIControlEnabled;
AttributeKey<Vec3f> attrTMI;
RDL2_DSO_ATTR_DEFINE(Map)
ImageMap_attr::attrWrapAround = sceneClass.declareAttribute<Bool>("wrap_around", true, FLAGS_NONE, INTERFACE_GENERIC, {"wrap around"});
sceneClass.setMetadata(ImageMap_attr::attrWrapAround, "label", "wrap around");
sceneClass.setMetadata(ImageMap_attr::attrWrapAround, "comment", "If true, the image will repeat itself at it's boundries.   If false, the image will be clamped at it's boundries.");
ImageMap_attr::attrTexture = sceneClass.declareAttribute<String>("texture", FLAGS_FILENAME, INTERFACE_GENERIC, {});
sceneClass.setMetadata(ImageMap_attr::attrTexture, "comment", "Filename that points to a texture .exr or .tx file (must be mip-mapped and tiled with maketx). If filename contains '<UDIM>', udim index substitution is performed on filename.  In the UDIM case, 'scale', 'offset', and 'wrap around' are ignored.");
ImageMap_attr::attrUseDefaultColor = sceneClass.declareAttribute<Bool>("use_default_color_when_missing", false, FLAGS_NONE, INTERFACE_GENERIC, {"use default color when missing"});
sceneClass.setMetadata(ImageMap_attr::attrUseDefaultColor, "label", "use default color when missing");
sceneClass.setMetadata(ImageMap_attr::attrUseDefaultColor, "comment", "Uses the 'default color' for missing udims and does not report error");
ImageMap_attr::attrDefaultColor = sceneClass.declareAttribute<Rgb>("default_color", Rgb(0.0f, 1.0f, 0.0f), FLAGS_NONE, INTERFACE_GENERIC, {"default color"});
sceneClass.setMetadata(ImageMap_attr::attrDefaultColor, "label", "default color");
sceneClass.setMetadata(ImageMap_attr::attrDefaultColor, "comment", "Default color to be used for missing udims when 'use default color when missing' is enabled");
ImageMap_attr::attrTextureEnum = sceneClass.declareAttribute<Int>("texture_coordinates", 0, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {"texture coordinates"});
sceneClass.setEnumValue(ImageMap_attr::attrTextureEnum, 0, "texture");
sceneClass.setEnumValue(ImageMap_attr::attrTextureEnum, 2, "input texture coordinates");
sceneClass.setEnumValue(ImageMap_attr::attrTextureEnum, 1, "hair surface");
sceneClass.setEnumValue(ImageMap_attr::attrTextureEnum, 3, "hair closest surface");
sceneClass.setMetadata(ImageMap_attr::attrTextureEnum, "label", "texture coordinates");
sceneClass.setMetadata(ImageMap_attr::attrTextureEnum, "comment", "Selects the source of the coordinates used to map the texture.  For meshes, the 'texture' setting will read a Vec2f attribute named 'surface_st' if it exists, otherwise it will use barycentric coordinates.   For points and curves, the 'texture' setting will read a Vec2f attribute named 'uv' if it exists.   If the attribute doesn't exist, parametric st coordinates are used for curves and a constant value of (1.0, 1.0) is used for points.  The 'input texture coordinates' setting will evaluate a binding on the 'input_texture_coordinates' parameter and use the resulting red and green channels as the texture coordinates.   The 'hair surface' setting will try to read an explicit attribute named 'surface_st'.   The 'hair closest surface' setting will try to read an explicit attribute named 'closest_surface_uv'.");
ImageMap_attr::attrAlphaOnly = sceneClass.declareAttribute<Bool>("alpha_only", false, FLAGS_NONE, INTERFACE_GENERIC, {"alpha only"});
sceneClass.setMetadata(ImageMap_attr::attrAlphaOnly, "label", "alpha only");
sceneClass.setMetadata(ImageMap_attr::attrAlphaOnly, "comment", "If true, the alpha channel of the texture will be placed in the rgb channels.  If the texture has no alpha channel, 1.0 is used, and the resulting texture lookup is then always white.");
ImageMap_attr::attrInputTextureCoordinate = sceneClass.declareAttribute<Vec3f>("input_texture_coordinates", Vec3f(0.0f, 0.0f, 0.0f), FLAGS_BINDABLE, INTERFACE_GENERIC, {"input texture coordinates"});
sceneClass.setMetadata(ImageMap_attr::attrInputTextureCoordinate, "label", "input texture coordinates");
sceneClass.setMetadata(ImageMap_attr::attrInputTextureCoordinate, "comment", "If the 'texture_coordinates' parameter is set to 'input texture coordinates' then this parameter is evaluated and the resulting red and green channels are used for the u and v coordinates respectively.");
ImageMap_attr::attrMipBias = sceneClass.declareAttribute<Float>("mip_bias", 0.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setGroup("Blur", ImageMap_attr::attrMipBias);
sceneClass.setMetadata(ImageMap_attr::attrMipBias, "label", "mip bias");
sceneClass.setMetadata(ImageMap_attr::attrMipBias, "comment", "Amount to offset derivatives which controls mipmap selection.   This can be used to provide a cheap, albiet low quality, blur.  Note: Setting this value to -1.0 effectively disables mip mapping and the highest resolution will be used.");
ImageMap_attr::attrGamma = sceneClass.declareAttribute<Int>("gamma", 2, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {});
sceneClass.setEnumValue(ImageMap_attr::attrGamma, 0, "off");
sceneClass.setEnumValue(ImageMap_attr::attrGamma, 1, "on");
sceneClass.setEnumValue(ImageMap_attr::attrGamma, 2, "auto");
sceneClass.setMetadata(ImageMap_attr::attrGamma, "comment", "If this is set to 'on' or 'auto' and the 'texture' file is 8-bit, then a power of 2.2 will be applied to the RGB channels of the image.");
ImageMap_attr::attrOffset = sceneClass.declareAttribute<Vec2f>("offset", Vec2f(0.0f, 0.0f), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(ImageMap_attr::attrOffset, "comment", "A constant offset applied to the texture coordinates.");
ImageMap_attr::attrScale = sceneClass.declareAttribute<Vec2f>("scale", Vec2f(1.0f, 1.0f), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(ImageMap_attr::attrScale, "comment", "A constant scale applied to the texture coordinates.");
ImageMap_attr::attrRotationAngle = sceneClass.declareAttribute<Float>("rotation_angle", 0.0f, FLAGS_NONE, INTERFACE_GENERIC, {"rotation angle"});
sceneClass.setMetadata(ImageMap_attr::attrRotationAngle, "label", "rotation angle");
sceneClass.setMetadata(ImageMap_attr::attrRotationAngle, "comment", "A constant rotation in degrees applied to the texture coordinates.");
ImageMap_attr::attrRotationCenter = sceneClass.declareAttribute<Vec2f>("rotation_center", Vec2f(0.5f, 0.5f), FLAGS_NONE, INTERFACE_GENERIC, {"rotation center"});
sceneClass.setMetadata(ImageMap_attr::attrRotationCenter, "label", "rotation center");
sceneClass.setMetadata(ImageMap_attr::attrRotationCenter, "comment", "UV coordinate around which to rotate");
ImageMap_attr::attrSaturationEnabled = sceneClass.declareAttribute<Bool>("saturation_enabled", false, FLAGS_NONE, INTERFACE_GENERIC, {"saturation enabled"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrSaturationEnabled);
sceneClass.setMetadata(ImageMap_attr::attrSaturationEnabled, "label", "saturation enabled");
sceneClass.setMetadata(ImageMap_attr::attrSaturationEnabled, "comment", "Enables 'saturation' parameter");
ImageMap_attr::attrSaturation = sceneClass.declareAttribute<Rgb>("saturation", Rgb(1.0f, 1.0f, 1.0f), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrSaturation);
sceneClass.setMetadata(ImageMap_attr::attrSaturation, "comment", "Applies saturation per channel with 0.0 being grayscale and 1.0 being full saturation");
ImageMap_attr::attrContrastEnabled = sceneClass.declareAttribute<Bool>("contrast_enabled", false, FLAGS_NONE, INTERFACE_GENERIC, {"contrast enabled"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrContrastEnabled);
sceneClass.setMetadata(ImageMap_attr::attrContrastEnabled, "label", "contrast enabled");
sceneClass.setMetadata(ImageMap_attr::attrContrastEnabled, "comment", "Enables 'contrast' parameter");
ImageMap_attr::attrContrast = sceneClass.declareAttribute<Rgb>("contrast", Rgb(1.0f, 1.0f, 1.0f), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrContrast);
sceneClass.setMetadata(ImageMap_attr::attrContrast, "comment", "Applies contrast per channel in the style of Nuke where the pivot is at 0.18");
ImageMap_attr::attrGammaEnabled = sceneClass.declareAttribute<Bool>("gamma_enabled", false, FLAGS_NONE, INTERFACE_GENERIC, {"gamma enabled"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrGammaEnabled);
sceneClass.setMetadata(ImageMap_attr::attrGammaEnabled, "label", "gamma enabled");
sceneClass.setMetadata(ImageMap_attr::attrGammaEnabled, "comment", "Enables 'gamma_adjust' parameter");
ImageMap_attr::attrGammaAdjust = sceneClass.declareAttribute<Rgb>("gamma_adjust", Rgb(1.0f, 1.0f, 1.0f), FLAGS_NONE, INTERFACE_GENERIC, {"gamma adjust"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrGammaAdjust);
sceneClass.setMetadata(ImageMap_attr::attrGammaAdjust, "label", "gamma adjust");
sceneClass.setMetadata(ImageMap_attr::attrGammaAdjust, "comment", "Applies gamma(power fuction) per channel.");
ImageMap_attr::attrGainOffsetEnabled = sceneClass.declareAttribute<Bool>("gain_offset_enabled", false, FLAGS_NONE, INTERFACE_GENERIC, {"gain offset enabled"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrGainOffsetEnabled);
sceneClass.setMetadata(ImageMap_attr::attrGainOffsetEnabled, "label", "gain offset enabled");
sceneClass.setMetadata(ImageMap_attr::attrGainOffsetEnabled, "comment", "Enables 'gain' and 'offset_adjust' parameters");
ImageMap_attr::attrGain = sceneClass.declareAttribute<Rgb>("gain", Rgb(1.0f, 1.0f, 1.0f), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrGain);
sceneClass.setMetadata(ImageMap_attr::attrGain, "comment", "Applies multiplier per channel.");
ImageMap_attr::attrOffsetAdjust = sceneClass.declareAttribute<Rgb>("offset_adjust", Rgb(0.0f, 0.0f, 0.0f), FLAGS_NONE, INTERFACE_GENERIC, {"offset adjust"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrOffsetAdjust);
sceneClass.setMetadata(ImageMap_attr::attrOffsetAdjust, "label", "offset adjust");
sceneClass.setMetadata(ImageMap_attr::attrOffsetAdjust, "comment", "Applies offset per channel.");
ImageMap_attr::attrTMIControlEnabled = sceneClass.declareAttribute<Bool>("TMI_control_enabled", false, FLAGS_NONE, INTERFACE_GENERIC, {"TME control enabled", "TMI control enabled"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrTMIControlEnabled);
sceneClass.setMetadata(ImageMap_attr::attrTMIControlEnabled, "label", "TMI control enabled");
sceneClass.setMetadata(ImageMap_attr::attrTMIControlEnabled, "comment", "Enables 'TMI' (Temperature Magenta Intensity) parameter");
ImageMap_attr::attrTMI = sceneClass.declareAttribute<Vec3f>("TMI", Vec3f(0.0f, 0.0f, 0.0f), FLAGS_NONE, INTERFACE_GENERIC, {"TME"});
sceneClass.setGroup("Color Correction", ImageMap_attr::attrTMI);
sceneClass.setMetadata(ImageMap_attr::attrTMI, "display", "tmi");
sceneClass.setMetadata(ImageMap_attr::attrTMI, "comment", "T = Temperature where positive values push towards blue and negative towards red.   M = Magenta where positive values push towards magenta and negative towards green.  I = Intensity where negative values remove and positive values add energy ");
RDL2_DSO_ATTR_END
using namespace ImageMap_attr;
