#include <scene_rdl2/scene/rdl2/rdl2.h>
using namespace scene_rdl2::rdl2;
RDL2_DSO_ATTR_DECLARE_NS(OpenVdbMap_attr)
AttributeKey<Int> attrVdbSource;
AttributeKey<String> attrTexture;
AttributeKey<SceneObject*> attrOpenVdbGeometry;
AttributeKey<String> attrGridName;
AttributeKey<Int> attrTextureCoordSource;
AttributeKey<Vec3f> attrInputTextureCoordinate;
AttributeKey<Int> attrInterpolation;
AttributeKey<Rgb> attrDefaultValue;
AttributeKey<Bool> attrShowWarnings;
AttributeKey<Bool> attrShowActiveField;
RDL2_DSO_ATTR_DEFINE(Map)
OpenVdbMap_attr::attrVdbSource = sceneClass.declareAttribute<Int>("vdb_source", 0, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {});
sceneClass.setEnumValue(OpenVdbMap_attr::attrVdbSource, 0, "from texture");
sceneClass.setEnumValue(OpenVdbMap_attr::attrVdbSource, 1, "from OpenVdbGeometry");
sceneClass.setMetadata(OpenVdbMap_attr::attrVdbSource, "label", "vdb source");
sceneClass.setMetadata(OpenVdbMap_attr::attrVdbSource, "comment", "Where to look for the vdb filename.  Choose 'from texture' to specify a .vdb filename directly via the 'texture' attribute.  Choose 'from OpenVdbGeometry' to use the .vdb filename and transform from an OpenVdbGeometry object in the scene using the 'openvdb geometry' attribute");
OpenVdbMap_attr::attrTexture = sceneClass.declareAttribute<String>("texture", FLAGS_FILENAME, INTERFACE_GENERIC, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrTexture, "label", "texture");
sceneClass.setMetadata(OpenVdbMap_attr::attrTexture, "comment", "");
OpenVdbMap_attr::attrOpenVdbGeometry = sceneClass.declareAttribute<SceneObject*>("openvdb_geometry", FLAGS_NONE, INTERFACE_GEOMETRY, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrOpenVdbGeometry, "label", "openvdb geometry");
sceneClass.setMetadata(OpenVdbMap_attr::attrOpenVdbGeometry, "comment", "The OpenVdbGeometry object from which to retrieve the .vdb filename and transform when 'vdb source' is set to 'from OpenVdbGeometry'");
OpenVdbMap_attr::attrGridName = sceneClass.declareAttribute<String>("grid_name", "", FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrGridName, "label", "grid name (optional)");
sceneClass.setMetadata(OpenVdbMap_attr::attrGridName, "comment", "The name of the grid within the .vdb file from which to sample (hint: use openvdb_print to see contents of .vdb file). If no grid is specified, the first grid found in the .vdb will be used.  In cases where there are multiple grids with the same name, the grid name can be indexed (eg. density[1])");
OpenVdbMap_attr::attrTextureCoordSource = sceneClass.declareAttribute<Int>("texture_coordinates", 0, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {});
sceneClass.setEnumValue(OpenVdbMap_attr::attrTextureCoordSource, 0, "position");
sceneClass.setEnumValue(OpenVdbMap_attr::attrTextureCoordSource, 1, "reference position");
sceneClass.setEnumValue(OpenVdbMap_attr::attrTextureCoordSource, 2, "input texture coordinates");
sceneClass.setMetadata(OpenVdbMap_attr::attrTextureCoordSource, "label", "texture coordinates");
sceneClass.setMetadata(OpenVdbMap_attr::attrTextureCoordSource, "comment", "Which coordinate source to use for the texture lookup");
OpenVdbMap_attr::attrInputTextureCoordinate = sceneClass.declareAttribute<Vec3f>("input_texture_coordinates", Vec3f(0.0f, 0.0f, 0.0f), FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrInputTextureCoordinate, "label", "input texture coordinates");
sceneClass.setMetadata(OpenVdbMap_attr::attrInputTextureCoordinate, "comment", "The world-space coordinate to use when 'texture coordinates' is set to 'input texture coordinates'");
OpenVdbMap_attr::attrInterpolation = sceneClass.declareAttribute<Int>("interpolation", 2, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {});
sceneClass.setEnumValue(OpenVdbMap_attr::attrInterpolation, 0, "point");
sceneClass.setEnumValue(OpenVdbMap_attr::attrInterpolation, 1, "box");
sceneClass.setEnumValue(OpenVdbMap_attr::attrInterpolation, 2, "quadratic");
sceneClass.setMetadata(OpenVdbMap_attr::attrInterpolation, "label", "interpolation");
sceneClass.setMetadata(OpenVdbMap_attr::attrInterpolation, "comment", "The type of interpolation to use when sampling the vdb");
OpenVdbMap_attr::attrDefaultValue = sceneClass.declareAttribute<Rgb>("default_value", Rgb(0,0,0), FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrDefaultValue, "label", "default value");
sceneClass.setMetadata(OpenVdbMap_attr::attrDefaultValue, "comment", "A default value to use when A) the .vdb file is not found, B) the requested grid is not found, C) the grid is unspecified, but no grid is found");
OpenVdbMap_attr::attrShowWarnings = sceneClass.declareAttribute<Bool>("show_warnings", false, FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(OpenVdbMap_attr::attrShowWarnings, "label", "show warnings");
sceneClass.setMetadata(OpenVdbMap_attr::attrShowWarnings, "comment", "Enables a warning message when A) the .vdb file is not found, B) the requested grid is not found, C) the grid is unspecified, but no grid is found");
OpenVdbMap_attr::attrShowActiveField = sceneClass.declareAttribute<Bool>("show_active_field", false, FLAGS_NONE, INTERFACE_GENERIC, {});
sceneClass.setGroup("Advanced", OpenVdbMap_attr::attrShowActiveField);
sceneClass.setMetadata(OpenVdbMap_attr::attrShowActiveField, "label", "show active field");
sceneClass.setMetadata(OpenVdbMap_attr::attrShowActiveField, "comment", "When enabled active/inactive field locations will be white/black, respectively");
RDL2_DSO_ATTR_END
using namespace OpenVdbMap_attr;
