#include <scene_rdl2/scene/rdl2/rdl2.h>
using namespace scene_rdl2::rdl2;
RDL2_DSO_ATTR_DECLARE_NS(UsdPreviewSurface_attr)
AttributeKey<Rgb> attrDiffuseColor;
AttributeKey<Rgb> attrEmissiveColor;
AttributeKey<Int> attrUseSpecularWorkflow;
AttributeKey<Rgb> attrSpecularColor;
AttributeKey<Float> attrMetallic;
AttributeKey<Float> attrRoughness;
AttributeKey<Float> attrClearcoat;
AttributeKey<Float> attrClearcoatRoughness;
AttributeKey<Float> attrOpacity;
AttributeKey<Float> attrOpacityThreshold;
AttributeKey<Float> attrIor;
AttributeKey<Vec3f> attrNormal;
AttributeKey<Float> attrDisplacement;
AttributeKey<Float> attrOcclusion;
RDL2_DSO_ATTR_DEFINE(Material)
UsdPreviewSurface_attr::attrDiffuseColor = sceneClass.declareAttribute<Rgb>("diffuseColor", Rgb(0.18f, 0.18f, 0.18f), FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrDiffuseColor, "comment", "When using metallic workflow this is interpreted as albedo.");
UsdPreviewSurface_attr::attrEmissiveColor = sceneClass.declareAttribute<Rgb>("emissiveColor", Rgb(0.0f, 0.0f, 0.0f), FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrEmissiveColor, "comment", "Emissive component.");
UsdPreviewSurface_attr::attrUseSpecularWorkflow = sceneClass.declareAttribute<Int>("useSpecularWorkflow", 0, FLAGS_ENUMERABLE, INTERFACE_GENERIC, {});
sceneClass.setEnumValue(UsdPreviewSurface_attr::attrUseSpecularWorkflow, 0, "Metalness workflow");
sceneClass.setEnumValue(UsdPreviewSurface_attr::attrUseSpecularWorkflow, 1, "Specular workflow");
sceneClass.setMetadata(UsdPreviewSurface_attr::attrUseSpecularWorkflow, "comment", "This node can fundamentally operate in two modes : Specular workflow where you provide a texture/value to the 'specularColor' input. Or, Metallic workflow where you provide a texture/value to the 'metallic' input.");
UsdPreviewSurface_attr::attrSpecularColor = sceneClass.declareAttribute<Rgb>("specularColor", Rgb(0.0f, 0.0f, 0.0f), FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrSpecularColor, "comment", "Specular color to be used. This is the color at 0 incidence. Edge color is assumed white. Transition between the two colors according to Schlick fresnel approximation.");
sceneClass.setMetadata(UsdPreviewSurface_attr::attrSpecularColor, "disable when", "{ useSpecularWorkflow == 'Metalness workflow' }");
UsdPreviewSurface_attr::attrMetallic = sceneClass.declareAttribute<Float>("metallic", 0.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrMetallic, "comment", "Use 1 for metallic surfaces and 0 for non-metallic.  If metallic is 1, then both F0 (reflectivity at 0 degree incidence) and edge F90 reflectivity will simply be the Albedo.  If metallic is 0, then Albedo is ignored in the calculation of F0 and F90; F0 is derived from ior via ( (1-ior)/(1+ior) )^2 and F90 is white. In between, we interpolate.");
sceneClass.setMetadata(UsdPreviewSurface_attr::attrMetallic, "disable when", "{ useSpecularWorkflow == 'Specular workflow' }");
UsdPreviewSurface_attr::attrRoughness = sceneClass.declareAttribute<Float>("roughness", 0.5f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrRoughness, "comment", "Roughness for the specular lobe. The value ranges from 0 to 1, which goes from a perfectly specular surface at 0.0 to maximum roughness of the specular lobe.");
UsdPreviewSurface_attr::attrClearcoat = sceneClass.declareAttribute<Float>("clearcoat", 0.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrClearcoat, "comment", "Second specular lobe amount. The color is white.");
UsdPreviewSurface_attr::attrClearcoatRoughness = sceneClass.declareAttribute<Float>("clearcoatRoughness", 0.01f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrClearcoatRoughness, "comment", "Roughness for the second specular lobe.");
UsdPreviewSurface_attr::attrOpacity = sceneClass.declareAttribute<Float>("opacity", 1.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrOpacity, "comment", "When opacity is 1.0 then the geometry is fully opaque, if it is smaller than 1.0 then the geometry is translucent, when it is 0 the geometry is transparent. Note that even a fully transparent object still receives lighting as, for example, perfectly clear glass still has a specular response.");
UsdPreviewSurface_attr::attrOpacityThreshold = sceneClass.declareAttribute<Float>("opacityThreshold", 0.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrOpacityThreshold, "comment", "The opacityThreshold input is useful for creating geometric cut-outs based on the opacity input. A value of 0.0 indicates that no masking is applied to the opacity input, while a value greater than 0.0 indicates that rendering of the surface is limited to the areas where the opacity is greater or equal to that value.");
UsdPreviewSurface_attr::attrIor = sceneClass.declareAttribute<Float>("ior", 1.5f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrIor, "comment", "Index of Refraction to be used for translucent objects and objects with specular components, including the clearcoat if clearcoat > 0.");
UsdPreviewSurface_attr::attrNormal = sceneClass.declareAttribute<Vec3f>("normal", Vec3f(0.0f, 0.0f, 1.0f), FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrNormal, "comment", "Expects normal in tangent space [(-1,-1,-1), (1,1,1)]");
UsdPreviewSurface_attr::attrDisplacement = sceneClass.declareAttribute<Float>("displacement", 0.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrDisplacement, "comment", "Displacement in the direction of the normal.");
UsdPreviewSurface_attr::attrOcclusion = sceneClass.declareAttribute<Float>("occlusion", 1.0f, FLAGS_BINDABLE, INTERFACE_GENERIC, {});
sceneClass.setMetadata(UsdPreviewSurface_attr::attrOcclusion, "comment", "Ignored by Moonray");
static const char *labels[] = {
    "diffuse",
    "specular",
    "specular transmission",
    "outer specular",
    nullptr
};
sceneClass.declareDataPtr("labels", labels);
RDL2_DSO_ATTR_END
using namespace UsdPreviewSurface_attr;
